#!/usr/bin/env python
###############################################################################
#  Dice.py -- die rolling utilities
###############################################################################

import whrandom

Select = whrandom.choice

def D(n, s, b = 0):
    res = b
    for i in range(n):
        res = res + whrandom.randint(1, s)
    return res

def Roll(table):
    "Roll() handles single-level tables"
    total = 0
    for i in table:
        total = total + i[0]
    sel = D(1, total, 0)
    for i in table:
        sel = sel - i[0]
        if sel < 1:
            return i
    return None # shouldn't get here.

def MRoll(table):
    """MRoll() handles nested tables

    The last element of the table row may be a list,
    in which case it is assumed to be a subtable,
    and is rolled on.

    The last element may also be a callable object;
    it receives the current row as a tuple argument, 
    and must return a "row" (tuple).

    Otherwise, the entire row is returned.
    """
    while table:
        row = Roll(table)
        # print "---", row[1]
        if type(row[-1]) is type([]):
            table = row[-1]
        else:
            try:
                rc = row[-1](row)
            except TypeError:
                rc = row
            row = rc
            table = None
    return row

# end of script.
